@@ -42,6 +42,7 @@ NamespacePtr	CairoImageNamespace;
 NamespacePtr	CairoPdfNamespace;
 NamespacePtr	CairoSvgNamespace;
 NamespacePtr	CairoPsNamespace;
+NamespacePtr	CairoRsvgNamespace;
 Type		*typeCairo;
 Type		*typeCairoSurface;
 Type		*typeCairoStatus;
@@ -69,6 +70,10 @@ Type		*typeCairoPatternExtend;
 Type		*typeCairoPatternFilter;
 Type		*typeCairoImageFormat;
 
+Type		*typeRsvg;
+Type		*typeRsvgDimensions;
+Type		*typeRsvgPosition;
+
 #define CAIRO_I		0
 #define CAIRO_S		"00"
 #define SURFACE_I	1
@@ -122,6 +127,15 @@ Type		*typeCairoImageFormat;
 #define FORMAT_I	32
 #define FORMAT_S	"32"
 
+#define RSVG_I		33
+#define RSVG_S		"33"
+
+#define RSVG_DIMENSIONS_I	34
+#define RSVG_DIMENSIONS_S	"34"
+
+#define RSVG_POSITION_I	35
+#define RSVG_POSITION_S	"35"
+
 static Type *
 make_typedef (char	*name_str,
 	      Namespace	*namespace,
@@ -440,6 +454,34 @@ init_types (void)
 				      NULL,
 				      typePrim[rep_foreign]);
 
+    CairoRsvgNamespace = BuiltinNamespace(&CairoNamespace, "Rsvg")->namespace.namespace;
+
+    typeRsvgDimensions = make_typedef("dimensions_t",
+				      CairoRsvgNamespace,
+				      publish_public,
+				      RSVG_DIMENSIONS_I,
+				      NULL,
+				      BuildStructType(4,
+						      typePrim[rep_int], "width",
+						      typePrim[rep_int], "height",
+						      typePrim[rep_float], "em",
+						      typePrim[rep_float], "ex"));
+    typeRsvgPosition = make_typedef("position_t",
+				      CairoRsvgNamespace,
+				      publish_public,
+				      RSVG_POSITION_I,
+				      NULL,
+				      BuildStructType(2,
+						      typePrim[rep_int], "x",
+						      typePrim[rep_int], "y"));
+
+    typeRsvg = make_typedef("rsvg_t",
+			    CairoRsvgNamespace,
+			    publish_public,
+			    RSVG_I,
+			    NULL,
+			    typePrim[rep_foreign]);
+			    
     EXIT();
 }
 
@@ -893,6 +935,12 @@ nickle_init (void)
 	    " void write_to_png (surface_t surface, file f)\n"
 	    "\n"
 	    " Write a surface to f in png format\n" },
+#if HAVE_CAIRO_XLIB_H
+	{ do_Cairo_Surface_set_window_shown, "set_window_shown", "v", SURFACE_S "b", "\n"
+	  " void set_window_shown (surface_t surface, bool shown)\n"
+	  "\n"
+	  " Show or hide the specified window\n" },
+#endif
 	{ 0 }
     };
     
@@ -901,7 +949,15 @@ nickle_init (void)
 	{ do_Cairo_Surface_create_window, "create_window", SURFACE_S, "snn", "\n"
 	    " surface_t create_window (real width, real height)\n"
 	    "\n"
-	    " Create a window and return a surface pointer for it\n" },
+	    " Create a window, show it, and return a surface pointer for it\n" },
+	{ do_Cairo_Surface_create_window_hidden, "create_window_hidden", SURFACE_S, "snn", "\n"
+	    " surface_t create_window_hidden (real width, real height)\n"
+	    "\n"
+	    " Create a window, but do not show it, and return a surface pointer for it\n" },
+	{ do_Cairo_Surface_resize_window, "resize_window", "v",  SURFACE_S "nn", "\n"
+	    " void resize_window (surface_t surface, real width, real height)\n"
+	    "\n"
+	    " Request that a window be resized. A configure event will signal the new size.\n" },
 #endif
 	{ do_Cairo_Surface_set_device_offset, "set_device_offset", "v", SURFACE_S "nn", "\n"
 	    " void set_device_offset (surface_t surface, real x, real y)\n"
@@ -1068,6 +1124,57 @@ nickle_init (void)
     };
 #endif
 					 
+    static const struct fbuiltin_1 rsvgfuncs_1[] = {
+	{ do_Rsvg_new_from_string, "new_from_string", RSVG_S, "s", "\n"
+	  " rsvg_t new_from_string (string svg)\n"
+	  "\n"
+	  " Create an RSVG object from the specified SVG string\n" },
+	{ do_Rsvg_new_from_file, "new_from_file", RSVG_S, "s", "\n"
+	  " rsvg_t new_from_file (string file)\n"
+	  "\n"
+	  " Create an RSVG object from the specified SVG file\n" },
+	{ do_Rsvg_get_dimensions, "get_dimensions", RSVG_DIMENSIONS_S, RSVG_S, "\n"
+	  " dimensions_t get_dimensions(rsvg_t rsvg)\n"
+	  "\n"
+	  " Get the dimensions and ex/em values for the specified SVG\n" },
+	{ do_Rsvg_get_title, "get_title", "s", RSVG_S, "\n"
+	  " string get_title(rsvg_t rsvg)\n"
+	  "\n"
+	  " Get the title of specified SVG\n" },
+	{ do_Rsvg_get_desc, "get_desc", "s", RSVG_S, "\n"
+	  " string get_desc(rsvg_t rsvg)\n"
+	  "\n"
+	  " Get the description of specified SVG\n" },
+	{ do_Rsvg_get_metadata, "get_metadata", "s", RSVG_S, "\n"
+	  " string get_metadata(rsvg_t rsvg)\n"
+	  "\n"
+	  " Get the metadata of specified SVG\n" },
+	{ 0 }
+    };
+
+    static const struct fbuiltin_2 rsvgfuncs_2[] = {
+	{ do_Rsvg_render, "render", "v", RSVG_S CAIRO_S, "\n"
+	  " void render (rsvg_t rsvg, cairo_t cairo)\n"
+	  "\n"
+	  " Draw an RSVG object using the specified cairo context\n" },
+	{ do_Rsvg_get_dimensions_sub, "get_dimensions_sub", RSVG_DIMENSIONS_S, RSVG_S "s", "\n"
+	  " dimensions_t get_dimensions_sub(rsvg_t rsvg, string id)\n"
+	  "\n"
+	  " Get the dimensions and ex/em values for the specified 'id' subset of an SVG\n" },
+	{ do_Rsvg_get_position_sub, "get_position_sub", RSVG_POSITION_S, RSVG_S "s", "\n"
+	  " position_t get_position_sub(rsvg_t rsvg, string id)\n"
+	  "\n"
+	  " Get the position of the specified 'id' subset of an SVG\n" },
+	{ 0 }
+    };
+
+    static const struct fbuiltin_3 rsvgfuncs_3[] = {
+	{ do_Rsvg_render_sub, "render_sub", "v", RSVG_S CAIRO_S "s", "\n"
+	  " void render_sub (rsvg_t rsvg, cairo_t cairo, string id)\n"
+	  "\n"
+	  " Draw the 'id' subset of an RSVG object using the specified cairo context\n" },
+	{ 0 }
+    };
     init_types ();
     
     BuiltinFuncs1 (&CairoNamespace, funcs_1);
@@ -1104,5 +1211,11 @@ nickle_init (void)
     BuiltinFuncs3 (&CairoPsNamespace, psfuncs_3);
 #endif
 
+    rsvg_init();
+
+    BuiltinFuncs1 (&CairoRsvgNamespace, rsvgfuncs_1);
+    BuiltinFuncs2 (&CairoRsvgNamespace, rsvgfuncs_2);
+    BuiltinFuncs3 (&CairoRsvgNamespace, rsvgfuncs_3);
+
     RETURN(TrueVal);
 }
